import unittest2 as unittest
from alice_pi import senders
import mock


class TestRestSender(unittest.TestCase):
    @mock.patch('requests.get',
                return_value=mock.MagicMock(status_code=404))
    def test_get_breaks_for_non_200(self, mock_get):
        sender = senders.RestSender('http://blah')

        with self.assertRaises(senders.DataNotSentException):
            sender.get(dict(test='a', test2='b'), {})

    @mock.patch('requests.post',
                return_value=mock.MagicMock(status_code=404))
    def test_post_breaks_for_non_200(self, mock_post):
        sender = senders.RestSender('http://blah')

        with self.assertRaises(senders.DataNotSentException):
            sender.post(dict(test='a', test2='b'), {})

    @mock.patch('requests.get',
                return_value=mock.MagicMock(status_code=200))
    def test_get_passes_200(self, mock_get):
        senders.RestSender('http://blah')

    @mock.patch('requests.post',
                return_value=mock.MagicMock(status_code=200))
    def test_post_passes_200(self, mock_post):
        senders.RestSender('http://blah')


if __name__ == '__main__':
    unittest.main()



import sys
import inspect
import requests

class DataNotSentException(Exception):
    pass

class Sender(object):
    """
    A Sender's responsibility is to send a measurement to a particular platform.
    """


class RestSender(Sender):

    def __init__(self, base_url, method='get', headers=None,
                 measurement_name=None, extra_params=None):

        send_callables = {'get': self.get,
                          'post': self.post}

        self.base_url = base_url
        self.method = send_callables[method]
        self.headers = headers

        if self.headers is None:
            self.headers = {}

        self.measurement_name = measurement_name
        self.extra_params = extra_params

    def __handle_response(self, response):
        status = response.status_code

        if status not in [200, 201]:
            raise DataNotSentException('Received {} from {}'.format(status,
                                                                    self.base_url))

    def send(self, measurement):
        meas_dict = {self.measurement_name: measurement}

        if self.extra_params is not None:
            meas_dict.update(self.extra_params)


        self.method(meas_dict, self.headers)

    def get(self, params, headers):
        response = requests.get(self.base_url,
                                params=params,
                                headers=headers)

        self.__handle_response(response)

    def post(self, params, headers):
        response = requests.post(self.base_url,
                                 json=params,
                                 headers=headers)
        self.__handle_response(response)

classes = inspect.getmembers(sys.modules[__name__],
                             inspect.isclass)

SENDERS = dict([tuple(item) for item in classes
                if item[1].__module__ == __name__ and
                'Exception' not in item[0]])



from alice_pi.senders import SENDERS
from alice_pi.measurers import MEASURERS
import yaml


class Configurer(object):
    def configure(self, config_path):

        config_dict = self.__load_config_dict(config_path)

        measurers = []
        for meas_group in config_dict.values():
            measurer = meas_group['measurer']

            measurer = MEASURERS[measurer]()

            senders = []
            for sender in meas_group['senders']:
                sender_name = next(iter(sender))
                sender_params = sender.get(sender_name)

                sender_obj = SENDERS[sender_name](**sender_params)
                senders.append(sender_obj)

            measurers.append(dict(measurer=measurer,
                                  senders=senders))
        return measurers

    def __load_config_dict(self, config_path):
        with open(config_path) as conf_file:
            config_dict = yaml.load(conf_file)

        return config_dict




import asyncio


class Manager(object):
    """
    A Manager's job is to manage collections of Measurer's and Senders
    """

    def __init__(self, measurers_and_senders):
        self.measurers_and_senders = measurers_and_senders

    def __init_measurements(self):
        for measurer_group in self.measurers_and_senders:
            measurer_group['measurement'] = None

    def run(self, count=None):
        """
        For every measurer, see if the value has changed, if it has,
        push to senders.
        """
        @asyncio.coroutine
        def check_status(measurer_group):
            new_status = measurer_group['measurer'].measure()

            if new_status != measurer_group['measurement']:
                return new_status
            else:
                return None

        @asyncio.coroutine
        def send_measurement(sender, measurement):
            sender.send(measurement)
            return

        @asyncio.coroutine
        def my_coroutine():
            loop_count = 0
            self.__init_measurements()
            while True:
                for measurer_group in self.measurers_and_senders:  # async for
                    meas_status = yield from check_status(measurer_group)
                    if meas_status is not None:
                        measurer_group['measurement'] = meas_status
                        for sender in measurer_group['senders']:  # async for
                            yield from send_measurement(sender,
                                                   measurer_group['measurement'])

                if count is not None:
                    loop_count += 1
                    if loop_count >= count:
                        break

        loop = asyncio.get_event_loop()
        loop.run_until_complete(my_coroutine())


if __name__ == '__main__':
    Manager('').run()



"""
A Measurer's responsibility is to interact with the sensor
hardware and take a measurement, then return that measurement
"""
import os
import sys
import inspect
from w1thermsensor import W1ThermSensor


if os.environ.get('PLATFORM', 'dev') == 'prod':
    import RPi.GPIO as GPIO


class Measurer(object):
    pass


class TemperatureMeasurer(Measurer):
    unit_dict = {'F': W1ThermSensor.DEGREES_F,
                 'C': W1ThermSensor.DEGREES_C}

    def __init__(self, unit='F', n_decimals=2):
        self.unit = self.unit_dict[unit]
        self.n_decimals = n_decimals

    def measure(self):
        sensor = W1ThermSensor()
        temp = sensor.get_temperature(self.unit)

        return round(temp, self.n_decimals)


class MotionMeasurer(Measurer):
    def measure():
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(11, GPIO.IN)  # Read output from PIR motion sensor

        return bool(GPIO.input(11))

classes = inspect.getmembers(sys.modules[__name__],
                             inspect.isclass)

MEASURERS = dict([tuple(item) for item in classes
                  if item[1].__module__ == __name__])

